# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: envoy/extensions/filters/http/aws_request_signing/v3/aws_request_signing.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AwsRequestSigningSigningAlgorithm",
    "AwsRequestSigning",
    "AwsRequestSigningQueryString",
    "AwsRequestSigningPerRoute",
)

import datetime
from dataclasses import dataclass

import betterproto2

from .......message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class AwsRequestSigningSigningAlgorithm(betterproto2.Enum):
    AWS_SIGV4 = 0
    """
    Use SigV4 for signing
    """

    AWS_SIGV4A = 1
    """
    Use SigV4A for signing
    """


@dataclass(eq=False, repr=False)
class AwsRequestSigning(betterproto2.Message):
    """
    [#protodoc-title: AwsRequestSigning]
    AwsRequestSigning :ref:`configuration overview <config_http_filters_aws_request_signing>`.
    [#extension: envoy.filters.http.aws_request_signing]

    Top level configuration for the AWS request signing filter.
    [#next-free-field: 9]
    """

    service_name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    The `service namespace
    <https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#genref-aws-service-namespaces>`_
    of the HTTP endpoint.

    Example: s3
    """

    region: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    Optional region string. If region is not provided, the region will be retrieved from the environment
    or AWS configuration files. See :ref:`config_http_filters_aws_request_signing_region` for more details.

    When signing_algorithm is set to ``AWS_SIGV4`` the region is a standard AWS `region <https://docs.aws.amazon.com/general/latest/gr/rande.html>`_ string for the service
    hosting the HTTP endpoint.

    Example: us-west-2

    When signing_algorithm is set to ``AWS_SIGV4A`` the region is used as a region set.

    A region set is a comma separated list of AWS regions, such as ``us-east-1,us-east-2`` or wildcard ``*``
    or even region strings containing wildcards such as ``us-east-*``

    Example: '*'

    By configuring a region set, a SigV4A signed request can be sent to multiple regions, rather than being
    valid for only a single region destination.
    """

    host_rewrite: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Indicates that before signing headers, the host header will be swapped with
    this value. If not set or empty, the original host header value
    will be used and no rewrite will happen.

    Note: this rewrite affects both signing and host header forwarding. However, this
    option shouldn't be used with
    :ref:`HCM host rewrite <envoy_v3_api_field_config.route.v3.RouteAction.host_rewrite_literal>` given that the
    value set here would be used for signing whereas the value set in the HCM would be used
    for host header forwarding which is not the desired outcome.
    """

    use_unsigned_payload: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)
    """
    Instead of buffering the request to calculate the payload hash, use the literal string ``UNSIGNED-PAYLOAD``
    to calculate the payload hash. Not all services support this option. See the `S3
    <https://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-header-based-auth.html>`_ policy for details.
    """

    match_excluded_headers: "list[_____type__matcher__v3__.StringMatcher]" = (
        betterproto2.field(5, betterproto2.TYPE_MESSAGE, repeated=True)
    )
    """
    A list of request header string matchers that will be excluded from signing. The excluded header can be matched by
    any patterns defined in the StringMatcher proto (e.g. exact string, prefix, regex, etc).

    Example:
    match_excluded_headers:
    - prefix: x-envoy
    - exact: foo
    - exact: bar
    When applied, all headers that start with "x-envoy" and headers "foo" and "bar" will not be signed.
    """

    signing_algorithm: "AwsRequestSigningSigningAlgorithm" = betterproto2.field(
        6,
        betterproto2.TYPE_ENUM,
        default_factory=lambda: AwsRequestSigningSigningAlgorithm(0),
    )
    """
    Optional Signing algorithm specifier, either ``AWS_SIGV4`` or ``AWS_SIGV4A``, defaulting to ``AWS_SIGV4``.
    """

    query_string: "AwsRequestSigningQueryString | None" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    If set, use the query string to store output of SigV4 or SigV4A calculation, rather than HTTP headers. The ``Authorization`` header will not be modified if ``query_string``
    is configured.

    Example:
    query_string: {}
    """

    credential_provider: "____common__aws__v3__.AwsCredentialProvider | None" = (
        betterproto2.field(8, betterproto2.TYPE_MESSAGE, optional=True)
    )
    """
    The credential provider for signing the request. This is optional and if not set,
    it will be retrieved from the procedure described in :ref:`config_http_filters_aws_request_signing`.
    """


default_message_pool.register_message(
    "envoy.extensions.filters.http.aws_request_signing.v3",
    "AwsRequestSigning",
    AwsRequestSigning,
)


@dataclass(eq=False, repr=False)
class AwsRequestSigningQueryString(betterproto2.Message):
    expiration_time: "datetime.timedelta | None" = betterproto2.field(
        1,
        betterproto2.TYPE_MESSAGE,
        unwrap=lambda: ______google__protobuf__.Duration,
        optional=True,
    )
    """
    Optional expiration time for the query string parameters. As query string parameter based requests are replayable, in effect representing
    an API call that has already been authenticated, it is recommended to keep this expiration time as short as feasible.
    This value will default to 5 seconds and has a maximum value of 3600 seconds (1 hour).
    """


default_message_pool.register_message(
    "envoy.extensions.filters.http.aws_request_signing.v3",
    "AwsRequestSigning.QueryString",
    AwsRequestSigningQueryString,
)


@dataclass(eq=False, repr=False)
class AwsRequestSigningPerRoute(betterproto2.Message):
    aws_request_signing: "AwsRequestSigning | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Override the global configuration of the filter with this new config.
    This overrides the entire message of AwsRequestSigning and not at field level.
    """

    stat_prefix: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    The human readable prefix to use when emitting stats.
    """


default_message_pool.register_message(
    "envoy.extensions.filters.http.aws_request_signing.v3",
    "AwsRequestSigningPerRoute",
    AwsRequestSigningPerRoute,
)


from .......google import protobuf as ______google__protobuf__
from ......type.matcher import v3 as _____type__matcher__v3__
from .....common.aws import v3 as ____common__aws__v3__
