# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: envoy/extensions/load_balancing_policies/ring_hash/v3/ring_hash.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "RingHashHashFunction",
    "RingHash",
)

import warnings
from dataclasses import dataclass

import betterproto2

from ......message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class RingHashHashFunction(betterproto2.Enum):
    """
    The hash function used to hash hosts onto the ketama ring.
    """

    DEFAULT_HASH = 0
    """
    Currently defaults to XX_HASH.
    """

    XX_HASH = 1
    """
    Use `xxHash <https://github.com/Cyan4973/xxHash>`_.
    """

    MURMUR_HASH_2 = 2
    """
    Use `MurmurHash2 <https://sites.google.com/site/murmurhash/>`_, this is compatible with
    std:hash<string> in GNU libstdc++ 3.4.20 or above. This is typically the case when compiled
    on Linux and not macOS.
    """


@dataclass(eq=False, repr=False)
class RingHash(betterproto2.Message):
    """
    [#protodoc-title: Ring Hash Load Balancing Policy]
    [#extension: envoy.load_balancing_policies.ring_hash]

    This configuration allows the built-in RING_HASH LB policy to be configured via the LB policy
    extension point. See the :ref:`load balancing architecture overview
    <arch_overview_load_balancing_types>` for more information.
    [#next-free-field: 8]
    """

    hash_function: "RingHashHashFunction" = betterproto2.field(
        1, betterproto2.TYPE_ENUM, default_factory=lambda: RingHashHashFunction(0)
    )
    """
    The hash function used to hash hosts onto the ketama ring. The value defaults to
    :ref:`XX_HASH<envoy_v3_api_enum_value_config.cluster.v3.Cluster.RingHashLbConfig.HashFunction.XX_HASH>`.
    """

    minimum_ring_size: "int | None" = betterproto2.field(
        2,
        betterproto2.TYPE_MESSAGE,
        unwrap=lambda: _____google__protobuf__.UInt64Value,
        optional=True,
    )
    """
    Minimum hash ring size. The larger the ring is (that is, the more hashes there are for each
    provided host) the better the request distribution will reflect the desired weights. Defaults
    to 1024 entries, and limited to 8M entries. See also
    :ref:`maximum_ring_size<envoy_v3_api_field_config.cluster.v3.Cluster.RingHashLbConfig.maximum_ring_size>`.
    """

    maximum_ring_size: "int | None" = betterproto2.field(
        3,
        betterproto2.TYPE_MESSAGE,
        unwrap=lambda: _____google__protobuf__.UInt64Value,
        optional=True,
    )
    """
    Maximum hash ring size. Defaults to 8M entries, and limited to 8M entries, but can be lowered
    to further constrain resource use. See also
    :ref:`minimum_ring_size<envoy_v3_api_field_config.cluster.v3.Cluster.RingHashLbConfig.minimum_ring_size>`.
    """

    use_hostname_for_hashing: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)
    """
    If set to ``true``, the cluster will use hostname instead of the resolved
    address as the key to consistently hash to an upstream host. Only valid for StrictDNS clusters with hostnames which resolve to a single IP address.

    .. note::
      This is deprecated and please use :ref:`consistent_hashing_lb_config
      <envoy_v3_api_field_extensions.load_balancing_policies.ring_hash.v3.RingHash.consistent_hashing_lb_config>` instead.
    """

    hash_balance_factor: "int | None" = betterproto2.field(
        5,
        betterproto2.TYPE_MESSAGE,
        unwrap=lambda: _____google__protobuf__.UInt32Value,
        optional=True,
    )
    """
    Configures percentage of average cluster load to bound per upstream host. For example, with a value of 150
    no upstream host will get a load more than 1.5 times the average load of all the hosts in the cluster.
    If not specified, the load is not bounded for any upstream host. Typical value for this parameter is between 120 and 200.
    Minimum is 100.

    This is implemented based on the method described in the paper https://arxiv.org/abs/1608.01350. For the specified
    ``hash_balance_factor``, requests to any upstream host are capped at ``hash_balance_factor/100`` times the average number of requests
    across the cluster. When a request arrives for an upstream host that is currently serving at its max capacity, linear probing
    is used to identify an eligible host. Further, the linear probe is implemented using a random jump in hosts ring/table to identify
    the eligible host (this technique is as described in the paper https://arxiv.org/abs/1908.08762 - the random jump avoids the
    cascading overflow effect when choosing the next host in the ring/table).

    If weights are specified on the hosts, they are respected.

    This is an O(N) algorithm, unlike other load balancers. Using a lower ``hash_balance_factor`` results in more hosts
    being probed, so use a higher value if you require better performance.

    .. note::
      This is deprecated and please use :ref:`consistent_hashing_lb_config
      <envoy_v3_api_field_extensions.load_balancing_policies.ring_hash.v3.RingHash.consistent_hashing_lb_config>` instead.
    """

    consistent_hashing_lb_config: "__common__v3__.ConsistentHashingLbConfig | None" = (
        betterproto2.field(6, betterproto2.TYPE_MESSAGE, optional=True)
    )
    """
    Common configuration for hashing-based load balancing policies.
    """

    locality_weighted_lb_config: "__common__v3__.LocalityLbConfigLocalityWeightedLbConfig | None" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Enable locality weighted load balancing for ring hash lb explicitly.
    """

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("use_hostname_for_hashing"):
            warnings.warn(
                "RingHash.use_hostname_for_hashing is deprecated", DeprecationWarning
            )
        if self.is_set("hash_balance_factor"):
            warnings.warn(
                "RingHash.hash_balance_factor is deprecated", DeprecationWarning
            )


default_message_pool.register_message(
    "envoy.extensions.load_balancing_policies.ring_hash.v3", "RingHash", RingHash
)


from ......google import protobuf as _____google__protobuf__
from ...common import v3 as __common__v3__
